#! /usr/bin/perl -w
#
# Set of common functions used in Nagios plugins
# Copyright (c) 2007 Emir Imamagic
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#
# Changes and Modifications
# =========================
# 21-Mar-2007 - Created; 
#               Parts of Nagios utils.pm are taken in order to make 
#               plugins Nagios-agnostic
# 

package sgutils;

use Sys::Hostname;
use Date::Format;

require Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = qw($SERVICEURI $SERVICETYPE $METRICNAME $SRM_PATH $GLOBUS_TCP_PORT_RANGE $GLOBUS_LOCATION $EDG_LOCATION $LCG_LOCATION $GLITE_LOCATION $TIMEOUT $VONAME $OUTPUT_TYPE @METRICS %ERRORS %OUTPUT_TYPES %COMMANDS &print_revision &support &printOutput &checkCommands &checkCommand &checkProxy &createProxy &printWLCGList &checkHost &checkEnvironment &checkMetric &print_revision_short);

#########################
##
##   CHANGEME: change software directories here!
##
#########################

# We should have /etc/java/java.conf with the location of java
sub find_java() {
    my $JAVA_CONF="/etc/java/java.conf";
    my $dir="";
    open(IN, "<$JAVA_CONF") || return "";
    while(<IN>) {
	next if /^\s*#/;
	if(/^\s*export\s*JAVA_HOME\s*=\s*([^\s]*)$/) {
	    $dir = $1;
	}
    }
    return $dir;
}

$GLOBUS_LOCATION = $ENV{GLOBUS_LOCATION} || "/opt/globus";
$GLITE_LOCATION = $ENV{GLITE_LOCATION} || "/opt/glite";
$LCG_LOCATION = $ENV{LCG_LOCATION} || "/opt/lcg";
$EDG_LOCATION = $ENV{EDG_LOCATION} || "/opt/edg";
$VDT_LOCATION = $ENV{VDT_LOCATION} || "/opt";
$SRM_PATH = $ENV{SRM_PATH} || "/opt/d-cache/srm";
$JAVA_HOME = $ENV{JAVA_HOME} || find_java();
$GLOBUS_TCP_PORT_RANGE = $ENV{GLOBUS_TCP_PORT_RANGE} || "20000,25000";
my $HOME = $ENV{HOME} || '/var/log/nagios';

#########################
#########################

#########################
#########################

#########################
##
##   Global variables
##
#########################

%ERRORS=(OK=>0,
            WARNING=>1,
            CRITICAL=>2,
            UNKNOWN=>3,
            DEPENDENT=>4);

%ERRORS_REV=(0=>'OK',
            1=>'WARNING',
            2=>'CRITICAL',
            3=>'UNKNOWN');            
            
%OUTPUT_TYPES=( NAGIOS=>0,
                WLCG=>1);
                    
$COMMANDS{GRID_PROXY_INFO} = "$GLOBUS_LOCATION/bin/grid-proxy-info";
$COMMANDS{GRID_PROXY_INIT} = "$GLOBUS_LOCATION/bin/grid-proxy-init";
$COMMANDS{VOMS_PROXY_INIT} = "$GLITE_LOCATION/bin/voms-proxy-init";
$COMMANDS{VOMS_PROXY_INFO} = "$GLITE_LOCATION/bin/voms-proxy-info";

$TIMEOUT = 60;
$VONAME = undef;
$OUTPUT_TYPE = undef;
@METRICS = undef;

$SERVICEURI = undef;
$SERVICETYPE = undef;
$METRICNAME = undef;
my $PROBESPECVER = '0.91';

#########################
#########################
 
#########################
##
##   Change default output type here!
##
#########################
$OUTPUT_TYPE = $OUTPUT_TYPES{WLCG};
 
#########################
##
##   Set environment variables based on known variables
##
#########################

$ENV{GLOBUS_LOCATION}=$GLOBUS_LOCATION;
$ENV{GLITE_LOCATION}=$GLITE_LOCATION;
$ENV{LCG_LOCATION}=$LCG_LOCATION;
$ENV{EDG_LOCATION} = $EDG_LOCATION;
$ENV{SRM_PATH}=$SRM_PATH;
$ENV{JAVA_HOME}=$JAVA_HOME;
$ENV{LD_LIBRARY_PATH}="$GLOBUS_LOCATION/lib:$GLITE_LOCATION/lib:$EDG_LOCATION/lib";
$ENV{GLOBUS_TCP_PORT_RANGE}=$GLOBUS_TCP_PORT_RANGE;
$ENV{PATH}="/usr/bin:/bin:$GLOBUS_LOCATION/bin:$GLITE_LOCATION/bin:$JAVA_HOME/bin";
$ENV{PYTHONPATH}="$GLITE_LOCATION/externals/lib/python2.2/site-packages:$GLITE_LOCATION/lib/python:$LCG_LOCATION/lib/python:$EDG_LOCATION/lib/python";
$ENV{HOME}=$HOME;
# Modifications to work on gLite 3.1 UI
$ENV{PYTHONPATH}="$ENV{PYTHONPATH}:/opt/fpconst/lib/python2.3/site-packages:/opt/SOAPpy/lib/python2.3/site-packages";
$ENV{LD_LIBRARY_PATH}="$ENV{LD_LIBRARY_PATH}:/opt/c-ares/lib:/opt/xerces-c/lib:/opt/log4cxx/lib:/opt/c-ares/lib:/opt/d-cache/dcap/lib";

#########################
#########################

#########################
##
##   Helper functions
##
#########################

sub checkCommands() {
    foreach my $command (values %COMMANDS) {
        checkCommand ($command);    
    }
    1;
}	

sub checkCommand ($) {
    my $command = shift;
    if (! -f $command) {
        printOutput ($ERRORS{UNKNOWN}, "Command $command doesn't exist.\n");
    }

    if (! -x $command) {
        printOutput ($ERRORS{UNKNOWN}, "Command $command is not executable.\n");
    }
    
    1;
}

sub checkEnvironment {
    foreach my $env (@_){
        unless ($ENV{$env}) {
            printOutput ($ERRORS{UNKNOWN}, "Environment variable $env must be set.\n");
        }
    }
    1;
}   

sub checkMetric
{
    my $metric = shift;
    my $retVal = 0;

    printOutput ($ERRORS{UNKNOWN}, "Metric must be defined. Supported metrics are: ".join(', ',@METRICS).".\n") unless ($metric);
        
    foreach my $m (@METRICS){
        if ($metric eq $m) {
            $retVal = 1;
            last;
        }
    }
   
    printOutput ($ERRORS{UNKNOWN}, "Unknown metric defined: $metric. Supported metrics are: ".join(', ',@METRICS).".\n") unless ($retVal);
    
    1;
}

sub checkHost {
    my $host = shift;

    if (defined $host) {
        if ($host !~ /^([-_.A-Za-z0-9]+\$?)$/) {
            printOutput ($ERRORS{UNKNOWN}, "Invalid host: $host.\n");
        }
    } else {
        printOutput ($ERRORS{UNKNOWN}, "Host must be defined.\n");
    }
    
    $host;
}

#########################
##
##   Grid proxy functions
##
#########################

sub checkProxy {
    my $proxy = shift;
    my $timeout = shift || "1:0";
    my $res;
    my $cmd;
    if (defined $proxy && -r $proxy) {
        $ENV{X509_USER_PROXY}=$proxy;
    }
    
    $cmd = $COMMANDS{GRID_PROXY_INFO} . " -e -valid $timeout 2>&1";

    $res = `$cmd`;
    if ($?) {
        return ($ERRORS{UNKNOWN}, "Valid grid proxy doesn't exist.\n", $res);
    }
    
    if ($VONAME) {
        $cmd = $COMMANDS{VOMS_PROXY_INFO} . " -vo 2>&1";
        $res = `$cmd`;
        if ($?) {
            return ($ERRORS{UNKNOWN}, "Error checking VOMS attributes.\n", $res);
        } else {
            unless ($res =~ /$VONAME/m) {
                return ($ERRORS{UNKNOWN}, "Proxy certificate doesn't have $VONAME VOMS attributes.\n", $res);
            }
        }
    }
    
    return ($ERRORS{OK}, "Grid proxy is valid.\n");
}

sub createProxy {
    my $pass = shift;
    my $vo = shift;
    my $passwd = "";
    my $res;
    my $cmd;

    if ($vo) {
        $cmd = "$COMMANDS{VOMS_PROXY_INIT} -voms $vo -pwstdin";
    } else {
        $cmd = "$COMMANDS{GRID_PROXY_INIT} -pwstdin";
    }
    
    if ($pass) {
        $cmd = "/bin/cat $pass | $cmd 2>&1";
    } else {
        $cmd = "/bin/echo | $cmd 2>&1";
    }
    
    $res = `$cmd`;
    if ($?) {
        return ($ERRORS{UNKNOWN}, "Valid grid proxy doesn't exist. Proxy creation failed.\n", $res);
    }
    $ERRORS{OK};
}

#########################
##
##   Output functions
##      (format output, print revision, etc)
##
#########################

sub print_revision ($$$) {
    my $plugin = shift;
    my $pluginRevision = shift;
    my $serviceVersion = shift;
    my $serviceT = $SERVICETYPE || '';
    my $probeV = $PROBESPECVER || '';
    print "$plugin\n";
    print "probeVersion: $pluginRevision\n";
    print "serviceType: $serviceT\n";
    print "serviceVersion: $serviceVersion\n";
    print "probeSpecificationVersion: $probeV\n\n";
    print "$plugin comes with ABSOLUTELY NO WARRANTY. 
This is free software, and you are welcome
to redistribute it under certain conditions.
";
}

sub print_revision_short ($$) {
    my $plugin = shift;
    my $pluginRevision = shift;
    print "$plugin $pluginRevision\n";
    print "$plugin comes with ABSOLUTELY NO WARRANTY. 
This is free software, and you are welcome
to redistribute it under certain conditions.
";
}


sub support() {
    print 'Send email to nagios-admin@srce.hr if you have questions
regarding use of this software or wish to submit patches or 
suggest improvements.
'; 
}

sub printOutput {
    my $status = shift;
    my $summary = shift;
    my $extra = shift;
    
    if ($OUTPUT_TYPE == $OUTPUT_TYPES{NAGIOS}) {
        printNagiosOutput($status,$summary);
    } elsif ($OUTPUT_TYPE == $OUTPUT_TYPES{WLCG}) {
        printWLCGOutput($status,$summary,$extra);
    }    
}

sub printNagiosOutput($$) {
    my $status = shift;
    my $summary = shift;

    print $summary;
    
    exit $status;
}

sub printWLCGOutput  {
    my $status = shift;
    my $summary = shift;
    my $extra = shift;
    my $hostname = hostname();
    my $serType = $SERVICETYPE || "unknown";
    my $metName = $METRICNAME || "unknown";
    my $statName = $ERRORS_REV{$status};
    
    print "serviceType: $serType\n";
    print "metricName: $metName\n";
    print "metricStatus: $statName\n";
    print "timestamp: ". time2str("%Y-%m-%dT%H:%M:%SZ",time(),'UTC') . "\n";
    print "summaryData: $summary";
    print "voName: $VONAME\n" if (defined $VONAME);
    if ($SERVICEURI) {
        print "serviceURI: $SERVICEURI\n";
        print "gatheredAt: $hostname\n";
    } else {
        print "hostName: $hostname\n";
    }
    if ($extra) {
        $extra =~ s/^\s*\n//mg;
        print "detailsData: $extra";
    }
    print "EOT\n";
    
    if ($status == $ERRORS{UNKNOWN}) {
        exit 1;
    } else {
        exit 0;
    }
}

sub printWLCGList  {
    foreach my $name (@METRICS) {
        print "serviceType: $SERVICETYPE\n";
        print "metricName: $name\n";
        print "metricType: status\n";
        print "EOT\n";
    }
}

1;
